@@ -15,6 +15,8 @@
 package v1alpha2
 
 import (
+	"context"
+
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	v1alpha2 "knative.dev/eventing/pkg/apis/sources/v1alpha2"
@@ -68,17 +70,20 @@ func newKnAPIServerSourcesClient(client clientv1alpha2.ApiServerSourceInterface,
 
 //GetAPIServerSource returns apiSource object if present
 func (c *apiServerSourcesClient) GetAPIServerSource(name string) (*v1alpha2.ApiServerSource, error) {
-	apiSource, err := c.client.Get(name, metav1.GetOptions{})
+	apiSource, err := c.client.Get(context.TODO(), name, metav1.GetOptions{})
 	if err != nil {
 		return nil, knerrors.GetError(err)
 	}
-
+	err = updateSourceGVK(apiSource)
+	if err != nil {
+		return nil, err
+	}
 	return apiSource, nil
 }
 
 //CreateAPIServerSource is used to create an instance of ApiServerSource
 func (c *apiServerSourcesClient) CreateAPIServerSource(apiSource *v1alpha2.ApiServerSource) error {
-	_, err := c.client.Create(apiSource)
+	_, err := c.client.Create(context.TODO(), apiSource, metav1.CreateOptions{})
 	if err != nil {
 		return knerrors.GetError(err)
 	}
@@ -88,7 +93,7 @@ func (c *apiServerSourcesClient) CreateAPIServerSource(apiSource *v1alpha2.ApiSe
 
 //UpdateAPIServerSource is used to update an instance of ApiServerSource
 func (c *apiServerSourcesClient) UpdateAPIServerSource(apiSource *v1alpha2.ApiServerSource) error {
-	_, err := c.client.Update(apiSource)
+	_, err := c.client.Update(context.TODO(), apiSource, metav1.UpdateOptions{})
 	if err != nil {
 		return knerrors.GetError(err)
 	}
@@ -98,8 +103,11 @@ func (c *apiServerSourcesClient) UpdateAPIServerSource(apiSource *v1alpha2.ApiSe
 
 //DeleteAPIServerSource is used to create an instance of ApiServerSource
 func (c *apiServerSourcesClient) DeleteAPIServerSource(name string) error {
-	err := c.client.Delete(name, &metav1.DeleteOptions{})
-	return err
+	err := c.client.Delete(context.TODO(), name, metav1.DeleteOptions{})
+	if err != nil {
+		return knerrors.GetError(err)
+	}
+	return nil
 }
 
 // Return the client's namespace
@@ -109,7 +117,7 @@ func (c *apiServerSourcesClient) Namespace() string {
 
 // ListAPIServerSource returns the available ApiServer type sources
 func (c *apiServerSourcesClient) ListAPIServerSource() (*v1alpha2.ApiServerSourceList, error) {
-	sourceList, err := c.client.List(metav1.ListOptions{})
+	sourceList, err := c.client.List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return nil, knerrors.GetError(err)
 	}
